local TwizUI = LibStub("AceAddon-3.0"):GetAddon("TwizUI")
local db, dbc, dbg

local TwizUICharacter_defaults = {
	initialized = false,
	needchatmoved = true,
}

local IWTextures = {
	Logo = [[Interface\AddOns\TwizUI\Media\Install\Logo.tga]],
}

local IWF = {}

---- Misc functions
TwizUI.deepCopyHash = function(t)
	local nt = {}
	for k, v in pairs(t) do
		if type(v) == "table" then
			nt[k] = TwizUI.deepCopyHash(v)
		else
			nt[k] = v
		end
	end
	return nt
end

-- Set default Chat frame position (called from Core.lua "PLAYER_ENTERING_WORLD")
function TwizUI:SetChatPosition()
	if TwizUICharacter.needchatmoved then
		ChatFrame1:ClearAllPoints()
		ChatFrame1:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", 6, 32)
		ChatFrame1:SetFrameLevel(15)
		ChatFrame1:SetHeight(145)
		ChatFrame1:SetWidth(400)
		ChatFrame1:SetUserPlaced(true)
		FCF_SavePositionAndDimensions(ChatFrame1)
		TwizUICharacter.needchatmoved = false
	end
end

-- CVars
local function SetDefaultCVars()
	-- Sound
	SetCVar("Sound_EnableErrorSpeech", 0)
	-- Nameplates
	SetCVar("bloatTest", 0)
	SetCVar("bloatnameplates", 0)
	SetCVar("bloatthreat", 0)
	-- Screenshots
	SetCVar("screenshotFormat", "jpg")				-- JPG format
	SetCVar("screenshotQuality", "10")				-- Highest quality
	-- Help
	SetCVar("showGameTips", 0)						-- Turn off Loading Screen Tips
	SetCVar("showTutorials", 0)						-- Turn off Tutorials
	SetCVar("UberTooltips", 1)						-- Turn on Enhanced Tooltips
	SetCVar("scriptErrors", 1)						-- Turn on Display Lua Errors
	-- Controls
	SetCVar("deselectOnClick", 1)					-- Turn off Sticky Targeting (inverted)
	-- Combat
	SetCVar("displaySpellActivationOverlays", 1)	-- Turn on Spell Alerts
	-- Display
	SetCVar("emphasizeMySpellEffects", 0)			-- Turn off Emphasize My Spell Effects
	SetCVar("SpellTooltip_DisplayAvgValues", 0)		-- Turn off Display Points As Average
	-- Social
	SetCVar("chatStyle", "classic")					-- Chat Style = "Classic"
	SetCVar("conversationMode", "inline")			-- Conversation Mode = "In-line"
	-- Quality of Life
	SetCVar("guildShowOffline", 0)					-- Hide Offline Guild Members
	SetCVar("profanityFilter", 0)					-- Turn off Profanity Filter
end

-- Initial Settings
local function InitialSettings()
	---- Chat
	-- Lock chat frames
	for i = 1,10 do
		local cf = _G["ChatFrame"..i]
		if cf then FCF_SetLocked(cf, 1) end
    end

	-- Set all chat channels to color player names by class
	for k, v in pairs(CHAT_CONFIG_CHAT_LEFT) do
		ToggleChatColorNamesByClassGroup(true, v.type)
	end
	for iCh = 1, 15 do
		ToggleChatColorNamesByClassGroup(true, "CHANNEL"..iCh)
	end

	-- Make Chat windows transparent
	SetChatWindowAlpha(1, 0)
	SetChatWindowAlpha(2, 0)
	
	-- Initial Settings done
	TwizUICharacter.initialized = true
end

---- Primary Installation
---- Stage 1
function TwizUI_RunStage1()
	dbc.installation.stage = -1
	
	if dbg.tags.firsttime then
		dbg.tags.firsttime = false
	end
	
	-- Make Chat windows transparent (again)
	SetChatWindowAlpha(1, 0)
	SetChatWindowAlpha(2, 0)
	
	-- Set TidyPlates theme to defaults
	if IsAddOnLoaded("TidyPlates") and IsAddOnLoaded("TidyPlates_ThreatPlates") then
		TidyPlatesOptions["primary"] = "Threat Plates"
		TidyPlatesOptions["secondary"] = "Threat Plates"
	end

	-- Reflux
	local RefluxArg = string.format("%s %s", "switch", "TwizUI")
    SlashCmdList.REFLUX(RefluxArg) -- This will cause a UI reload
end

local function CreateIWTextureFrame(texture, width, height, position, color)
	local frame = CreateFrame("Frame", nil, IWF)
	frame:SetParent(IWF)
	frame:SetPoint(unpack(position))
	frame:SetFrameStrata("DIALOG")
	frame:SetFrameLevel(IWF:GetFrameLevel() + 1)
	frame:SetWidth(width)
	frame:SetHeight(height)
	
	frame.bg = frame:CreateTexture()
	frame.bg:SetAllPoints(frame)
	frame.bg:SetTexture(texture)
	frame.bg:SetVertexColor(unpack(color))
	
	return frame
end

local function CreateInstallWindow()
	-- Background
	IWF = CreateFrame("Frame", nil, UIParent)
	IWF:SetParent(UIParent)
	IWF:SetAllPoints(UIParent)
	IWF:SetFrameStrata("DIALOG")
	IWF:SetFrameLevel(0)
	IWF:SetBackdrop({
		bgFile = "Interface\\Buttons\\WHITE8X8",
	})
	IWF:SetBackdropColor(0, 0, 0, 0.9)
	
	-- Button
	IWF.install = CreateFrame("Button", "TwizUI_Install", IWF, "SecureActionButtonTemplate")
	IWF.install:SetPoint("CENTER", IWF, "CENTER", 0, 0)
	IWF.install:SetWidth(110)
	IWF.install:SetHeight(50)
	IWF.install:SetNormalFontObject(NumberFontNormal)
	IWF.install:SetText("INSTALL")
	IWF.install:RegisterForClicks("LeftButtonUp")
	IWF.install:SetScript("OnClick", function()
		TwizUI_RunStage1()
	end)
	
	if Aurora then
		local F = unpack(Aurora)
		F.Reskin(IWF.install)
		
		-- Button Highlight
		IWF.installhighlight = CreateFrame("Frame", nil, IWF.install)
		IWF.installhighlight:SetPoint("CENTER", IWF.install, "CENTER", 0, 0)
		IWF.installhighlight:SetWidth(108)
		IWF.installhighlight:SetHeight(48)
		IWF.installhighlight:SetBackdrop({
			bgFile = "Interface\\Buttons\\WHITE8X8", 
			edgeFile = "Interface\\Buttons\\WHITE8X8", 
			tile = false, tileSize = 0, edgeSize = 1, 
			insets = { left = 1, right = 1, top = 1, bottom = 1	}
		})
		IWF.installhighlight:SetBackdropColor(0,0,0,0.1)
		IWF.installhighlight:SetBackdropBorderColor(RAID_CLASS_COLORS[TwizUI.class].r, RAID_CLASS_COLORS[TwizUI.class].g, RAID_CLASS_COLORS[TwizUI.class].b)
	end
	
	-- Logo
	IWF.logo = CreateIWTextureFrame(IWTextures.Logo, 256, 64, {"CENTER", IWF, "CENTER", 0, 60}, {1, 1, 1, 1})
end

local function Hide_TPTP()
	if IsAddOnLoaded("TidyPlates_ThreatPlates") then
		hooksecurefunc("StaticPopup_Show", function(...)
			if StaticPopupDialogs["SetToThreatPlates"] ~= nil then StaticPopup_Hide("SetToThreatPlates") end
		end)
	end
end

local function InstallationStage1()
	---- Create Installation Window
	CreateInstallWindow()
	
	---- First Time
	if dbg.tags.firsttime then
		-- CVars
		SetDefaultCVars()
	end
	
	---- Initial Character Settings
	if not TwizUICharacter.initialized then
		Hide_TPTP()
		if StaticPopup1 then StaticPopup1:Hide() end
		InitialSettings()
	end
	
	DEFAULT_CHATFRAME_ALPHA = 0
end

---- Process
local function PrimaryInstallation()
	if dbc.installation.stage > -1 then
		InstallationStage1()
	end
end

---- Install Procedure
function TwizUI:InstallProcedure()
	db = self.db.profile
	dbc = self.db.char
	dbg = self.db.global
	
	-- Set Char defaults
	if not(TwizUICharacter) then
		TwizUICharacter = TwizUICharacter_defaults
	end
	
	-- Primary Stages
	if dbc.installation.stage > -1 then
		PrimaryInstallation()
	end
end